package net.cattaka.android.variins;

import net.cattaka.android.variins.bean.ToneInfo;
import net.cattaka.android.variins.utils.ToneUtil;
import android.app.Activity;
import android.graphics.drawable.Drawable;
import android.os.Bundle;
import android.view.MotionEvent;
import android.view.View;
import android.widget.ImageView;

public class VariIns extends Activity {
	private ImageView keyboardView;
	private ToneUtil toneUtil;
	
	/** Called when the activity is first created. */
	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.main);

		Drawable keyboardImage = getResources().getDrawable(R.drawable.keyboardx2);
		keyboardView = (ImageView)findViewById(R.id.KeyboardView);
		
		keyboardView.setBackgroundDrawable(keyboardImage);
		keyboardView.setOnTouchListener(new View.OnTouchListener() {
			@Override
			public boolean onTouch(View v, MotionEvent event) {
				if (event.getAction() == MotionEvent.ACTION_DOWN || event.getAction() == MotionEvent.ACTION_MOVE) {
					//Log.d("onTouch",event.toString());
					playTone(createToneInfo(event));
					return true;
				} else if (event.getAction() == MotionEvent.ACTION_UP) {
					stopTone();
					return true;
				} else {
					return false;
				}
			}
		});
	}
	
	@Override
	protected void onResume() {
		super.onResume();
		this.toneUtil = new ToneUtil();
		this.toneUtil.init();
	}
	
	@Override
	protected void onPause() {
		this.toneUtil.dispose();
		this.toneUtil = null;
		super.onPause();
	}
	
	private ToneInfo createToneInfo(MotionEvent event) {
		float w = keyboardView.getWidth();
		float h = keyboardView.getHeight();
		
		float freqF = 2 * event.getX() / w; 
		float volume = 1 - event.getY() / h;
		
		volume = (volume < 0) ? 0 : ((volume > 1) ? 1 : volume);
		
		int freq = (int)((double)ToneInfo.DEFAULT_FREQUENCY * Math.pow(2, freqF));
		
		return new ToneInfo(freq, volume);
	}
	private void playTone(ToneInfo toneInfo) {
		//Log.d("VariIns", toneInfo.toString());
		if (toneUtil != null) {
			toneUtil.playTone(toneInfo);
		}
	}
	private void stopTone() {
		if (toneUtil != null) {
			toneUtil.stopTone();
		}
	}
}