package net.cattaka.android.variins.utils;

import net.cattaka.android.variins.bean.ToneInfo;
import android.media.AudioFormat;
import android.media.AudioManager;
import android.media.AudioTrack;
import android.util.Log;

public class ToneUtil {
	private static final int DEFAUIT_SAMPLING_RATE = 11025;
	private static final float DEFAULT_DURATION = 0.01f;
	private AudioThread audioThread;
	
	class AudioThread extends Thread {
		private AudioTrack audioTrack;
		private Object syncKey = new Object();
		private volatile boolean disposeFlag = false;
		private volatile byte[] currentAudioData;
		private volatile ToneInfo currentToneInfo;
		private volatile ToneInfo nextToneInfo;
		
		public AudioThread() {
			super("AudioThread");
			currentToneInfo = new ToneInfo();

			int suggestBufferSize = (int)(DEFAUIT_SAMPLING_RATE * DEFAULT_DURATION);
			int bufferSize = android.media.AudioTrack.getMinBufferSize(
					DEFAUIT_SAMPLING_RATE,
					AudioFormat.CHANNEL_CONFIGURATION_MONO,
					AudioFormat.ENCODING_PCM_8BIT);
			if (suggestBufferSize * 2 > bufferSize) {
				bufferSize = suggestBufferSize * 2;
			}
			
			int bufferSizeInBytes = bufferSize;
			audioTrack = new AudioTrack(
					AudioManager.STREAM_MUSIC,
					DEFAUIT_SAMPLING_RATE,
					AudioFormat.CHANNEL_CONFIGURATION_MONO,
					AudioFormat.ENCODING_PCM_8BIT,
					bufferSizeInBytes,
					AudioTrack.MODE_STREAM);
			
			currentAudioData = new byte[suggestBufferSize];
			for (int i=0;i<currentAudioData.length;i++) {
				currentAudioData[i] = 127;
			}
		}
		
		@Override
		public void run() {
			disposeFlag = false;
			audioTrack.play();
			float rad = 0;
			
			while(true) {
				audioTrack.write(currentAudioData, 0, currentAudioData.length);
				synchronized (syncKey) {
					if (disposeFlag) {
						break;
					}
					if (nextToneInfo != null && !currentToneInfo.equals(nextToneInfo)) {
						currentToneInfo.set(nextToneInfo);
						nextToneInfo = null;
					}
					if (currentToneInfo.isMute()) {
						for (int i=0;i<currentAudioData.length;i++) {
							currentAudioData[i] = (byte)0xFF;
						}
						rad = 0;
					} else {
						float dt = 1f / DEFAUIT_SAMPLING_RATE;
						float l = 1f / currentToneInfo.getFrequency();
						for (int i=0;i<currentAudioData.length;i++) {
							rad += 2 * Math.PI * dt / l;
							if (rad >= 2 * Math.PI) {
								rad -= 2 * Math.PI;
							}
							currentAudioData[i] = (byte)(127.0 + Math.sin(rad) * 127 * currentToneInfo.getVolume());
						}
					}
				}
			}
		}
		
		public void dispose() {
			synchronized (syncKey) {
				disposeFlag = true;
				//audioTrack.stop();
			}
		}
		
		public void playAudioData(ToneInfo toneInfo) {
			synchronized (syncKey) {
				nextToneInfo = toneInfo;
				//audioTrack.stop();
			}
		}
		public void stopAudioData() {
			synchronized (syncKey) {
				nextToneInfo = new ToneInfo();
				//audioTrack.stop();
			}
		}
	}
	
	/**
	 * 
	 */
	public ToneUtil() {
		// TODO
	}
	public void init() {
		audioThread = new AudioThread();
		audioThread.start();
	}

	public void playTone(ToneInfo toneInfo) {
		audioThread.playAudioData(toneInfo);
	}
	
	public void stopTone() {
		audioThread.stopAudioData();
	}

	public void dispose() {
		if (audioThread != null) {
			audioThread.dispose();
			try {
				audioThread.join();
			} catch (InterruptedException e) {
				Log.e("VariIns", "ToneUtil#dispose",e);
			}
		}
		audioThread = null;
	}
}
